#include "stdafx.h"
#include "MFCSensorApp.h"
#include "MFCSensorAppDlg.h"
#include "afxdialogex.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#endif


#pragma warning(suppress: 26433 26436)
STDMETHODIMP CMySensorEvents::OnDataUpdated(__RPC__in_opt ISensor* /*pSensor*/, __RPC__in_opt ISensorDataReport* pNewData) //NOLINT(clang-diagnostic-microsoft-exception-spec)
{
  //Attach our helper class which makes working with location data reports easier
  SensorAPI::CLocationSensorDataReport gpsDataReport{pNewData};

  //Get a pointer back to the dialog class so that we can display all the updated values
#pragma warning(suppress: 26466)
  const CMFCSensorAppDlg* pDlg{static_cast<CMFCSensorAppDlg*>(AfxGetMainWnd())};
#pragma warning(suppress: 26496)
  AFXASSUME(pDlg != nullptr);
  ASSERT(pDlg->IsKindOf(RUNTIME_CLASS(CMFCSensorAppDlg)));

  //Get the Latitude
  CString sMessage;
  double dLatitude{0};
  HRESULT hr{gpsDataReport.GetLatitude(dLatitude)};
  if (SUCCEEDED(hr))
    sMessage.Format(_T("%f"), dLatitude);
  else
    sMessage = _T("Failed");
  pDlg->GetDlgItem(IDC_LATITUDE)->SetWindowText(sMessage); //NOLINT(clang-analyzer-core.CallAndMessage)

  //Get the Longitude
#pragma warning(suppress: 26477)
  ATLASSERT(SUCCEEDED(hr));
  double dLongitude{0};
  hr = gpsDataReport.GetLongitude(dLongitude);
  if (SUCCEEDED(hr))
    sMessage.Format(_T("%f"), dLongitude);
  else
    sMessage = _T("Failed");
  pDlg->GetDlgItem(IDC_LONGITUDE)->SetWindowText(sMessage);

  //Get the sea level altitude
  double dAltitude{0};
  hr = gpsDataReport.GetSealevelAltitude(dAltitude);
  if (SUCCEEDED(hr))
    sMessage.Format(_T("%f meters"), dAltitude);
  else
    sMessage = _T("Failed");
  pDlg->GetDlgItem(IDC_SEALEVEL_ALTITUDE)->SetWindowText(sMessage);

  int nSatellitesInView{0};
  hr = gpsDataReport.GetSatellitesInView(nSatellitesInView);
  if (SUCCEEDED(hr))
    sMessage.Format(_T("%d"), nSatellitesInView);
  else
    sMessage = _T("Failed");
  pDlg->GetDlgItem(IDC_SATELLITES_IN_VIEW)->SetWindowText(sMessage);

  double dHDOP{0};
  hr = gpsDataReport.GetHDOP(dHDOP);
  if (SUCCEEDED(hr))
    sMessage.Format(_T("%f"), dHDOP);
  else
    sMessage = _T("Failed");
  pDlg->GetDlgItem(IDC_HDOP)->SetWindowText(sMessage);

  double dVDOP{0};
  hr = gpsDataReport.GetVDOP(dVDOP);
  if (SUCCEEDED(hr))
    sMessage.Format(_T("%f"), dVDOP);
  else
    sMessage = _T("Failed");
  pDlg->GetDlgItem(IDC_VDOP)->SetWindowText(sMessage);

  double dPDOP{0};
  hr = gpsDataReport.GetPDOP(dPDOP);
  if (SUCCEEDED(hr))
    sMessage.Format(_T("%f"), dPDOP);
  else
    sMessage = _T("Failed");
  pDlg->GetDlgItem(IDC_PDOP)->SetWindowText(sMessage);

  int nSatellitesUsed{0};
  hr = gpsDataReport.GetSatellitesUsed(nSatellitesUsed);
  if (SUCCEEDED(hr))
    sMessage.Format(_T("%d"), nSatellitesUsed);
  else
    sMessage = _T("Failed");
  pDlg->GetDlgItem(IDC_SATELLITES_USED)->SetWindowText(sMessage);

  double dSpeed{0};
  hr = gpsDataReport.GetSpeed(dSpeed);
  if (SUCCEEDED(hr))
    sMessage.Format(_T("%f knots"), dSpeed);
  else
    sMessage = _T("Failed");
  pDlg->GetDlgItem(IDC_SPEED)->SetWindowText(sMessage);

  int nStatus{0};
  hr = gpsDataReport.GetGPSStatus(nStatus);
  if (SUCCEEDED(hr))
    sMessage.Format(_T("%d"), nStatus);
  else
    sMessage = _T("Failed");
  pDlg->GetDlgItem(IDC_STATUS)->SetWindowText(sMessage);

  int nOperationMode{0};
  hr = gpsDataReport.GetGPSOperationMode(nOperationMode);
  if (SUCCEEDED(hr))
    sMessage.Format(_T("%d"), nOperationMode);
  else
    sMessage = _T("Failed");
  pDlg->GetDlgItem(IDC_OPERATION_MODE)->SetWindowText(sMessage);

  int nFixQuality{0};
  hr = gpsDataReport.GetFixQuality(nFixQuality);
  if (SUCCEEDED(hr))
    sMessage.Format(_T("%d"), nFixQuality);
  else
    sMessage = _T("Failed");
  pDlg->GetDlgItem(IDC_FIX_QUALITY)->SetWindowText(sMessage);

  int nDataTypeFixType{0};
  hr = gpsDataReport.GetDataTypeFixType(nDataTypeFixType);
  if (SUCCEEDED(hr))
    sMessage.Format(_T("%d"), nDataTypeFixType);
  else
    sMessage = _T("Failed");
  pDlg->GetDlgItem(IDC_DATA_TYPE_FIX)->SetWindowText(sMessage);

  int nSelectionMode{0};
  hr = gpsDataReport.GetGPSSelectionMode(nSelectionMode);
  if (SUCCEEDED(hr))
    sMessage.Format(_T("%d"), nSelectionMode);
  else
    sMessage = _T("Failed");
  pDlg->GetDlgItem(IDC_SELECTION_MODE)->SetWindowText(sMessage);

  double dTrueHeading{0};
  hr = gpsDataReport.GetTrueHeading(dTrueHeading);
  if (SUCCEEDED(hr))
    sMessage.Format(_T("%f"), dTrueHeading);
  else
    sMessage = _T("Failed");
  pDlg->GetDlgItem(IDC_TRUE_HEADING)->SetWindowText(sMessage);

#ifdef _DEBUG
  double dGeoidalSeparation{0};
  hr = gpsDataReport.GetGeoidalSeparation(dGeoidalSeparation);
#pragma warning(suppress: 26477)
  ATLASSERT(SUCCEEDED(hr));
  double dErrorRadius{0};
  hr = gpsDataReport.GetErrorRadius(dErrorRadius);
#pragma warning(suppress: 26477)
  ATLASSERT(SUCCEEDED(hr));
  std::vector<double> satellitesInViewAzimuth;
  hr = gpsDataReport.GetSatellitesInViewAzimuth(satellitesInViewAzimuth);
#pragma warning(suppress: 26477)
  ATLASSERT(SUCCEEDED(hr));
  std::vector<double> satellitesInViewElevation;
  hr = gpsDataReport.GetSatellitesInViewAzimuth(satellitesInViewElevation);
#pragma warning(suppress: 26477)
  ATLASSERT(SUCCEEDED(hr));
  std::vector<ULONG> satellitesUsedPRNS;
  hr = gpsDataReport.GetSatellitesUsedPRNS(satellitesUsedPRNS);
#pragma warning(suppress: 26477)
  ATLASSERT(SUCCEEDED(hr));
  std::vector<ULONG> satelliteIds;
  hr = gpsDataReport.GetSatellitesInViewIDs(satelliteIds); //NOLINT(clang-analyzer-deadcode.DeadStores)
  std::vector<double> satellitesInViewSTNRatio;
  hr = gpsDataReport.GetSatellitesInViewSTNRatio(satellitesInViewSTNRatio);
#pragma warning(suppress: 26477)
  ATLASSERT(SUCCEEDED(hr));
  double dMagneticHeading{0};
  hr = gpsDataReport.GetMagneticHeading(dMagneticHeading); //NOLINT(clang-analyzer-deadcode.DeadStores)
  std::wstring sSentence;
  hr = gpsDataReport.GetNMEASentence(sSentence);
#pragma warning(suppress: 26477)
  ATLASSERT(SUCCEEDED(hr));
#endif //#ifdef _DEBUG

  return S_OK;
}


#pragma warning(suppress: 26433 26440)
BEGIN_MESSAGE_MAP(CMFCSensorAppDlg, CDialog) //NOLINT(modernize-avoid-c-arrays)
  ON_WM_PAINT()
  ON_WM_QUERYDRAGICON()
END_MESSAGE_MAP()

#pragma warning(suppress: 26455)
CMFCSensorAppDlg::CMFCSensorAppDlg(_In_opt_ CWnd* pParent) : CDialog{CMFCSensorAppDlg::IDD, pParent},
                                                             m_hIcon{AfxGetApp()->LoadIcon(IDR_MAINFRAME)}
{
}

BOOL CMFCSensorAppDlg::OnInitDialog()
{
  //Let the base class do its thing
  __super::OnInitDialog();

  //Set the icon for this dialog
  SetIcon(m_hIcon, TRUE);
  SetIcon(m_hIcon, FALSE);

  //Instantiate the sensor manager on the local machine
  SensorAPI::CSensorManager<> manager;
  HRESULT hr{manager.Initialize()}; //NOLINT(clang-analyzer-deadcode.DeadStores)
#pragma warning(suppress: 26477)
  ATLASSERT(SUCCEEDED(hr));
  hr = manager.SetEventSink();
  if (FAILED(hr))
  {
    AfxMessageBox(_T("Could not create the Sensor Manager. Please make sure you are running on Windows 7 or later"), MB_OK | MB_ICONSTOP);
    EndDialog(IDCANCEL);
    return FALSE;
  }

  //Get all the GPS sensors on the local machine
  ATL::CComPtr<ISensorCollection> sensorCollection;
  hr = manager->GetSensorsByType(SENSOR_TYPE_LOCATION_GPS, &sensorCollection);
  if (FAILED(hr))
  {
    AfxMessageBox(_T("Could not find any GPS sensors on this machine"), MB_OK | MB_ICONSTOP);
    EndDialog(IDCANCEL);
    return FALSE;
  }

  //In this sample app, lets just use the first GPS sensor in the collection
  ULONG nCount{0};
  hr = sensorCollection->GetCount(&nCount);
  if (FAILED(hr) || (nCount == 0))
  {
    AfxMessageBox(_T("There are no GPS sensors installed on this machine"), MB_OK | MB_ICONSTOP);
    EndDialog(IDCANCEL);
    return FALSE;
  }
  hr = sensorCollection->GetAt(0, &m_Sensor);
  if (FAILED(hr))
  {
    AfxMessageBox(_T("Could not connect to first GPS sensor on this machine"), MB_OK | MB_ICONSTOP);
    EndDialog(IDCANCEL);
    return FALSE;
  }

#ifdef _DEBUG
  std::wstring sFriendlyName;
  hr = m_Sensor.GetFriendlyName(sFriendlyName); //NOLINT(clang-analyzer-deadcode.DeadStores)
#endif //#ifdef _DEBUG

  //Lets check to make sure it is not disabled and if so request permission to use this sensor
  SensorState state;
  hr = m_Sensor->GetState(&state);
  if (SUCCEEDED(hr) && (state == SENSOR_STATE_ACCESS_DENIED))
  {
    //Request permission to use this sensor
    hr = manager->RequestPermissions(GetSafeHwnd(), sensorCollection, TRUE);
    if ((hr == HRESULT_FROM_WIN32(ERROR_ACCESS_DENIED)) || (hr == HRESULT_FROM_WIN32(ERROR_CANCELLED)))
    {
      AfxMessageBox(_T("Please use the Windows Location settings applet to enable your GPS sensor and run this program again"), MB_OK | MB_ICONSTOP);
      EndDialog(IDCANCEL);
      return FALSE;
    }
  }

  //Hook up events for the sensor
  hr = m_Sensor.SetEventSink();
  if (FAILED(hr))
  {
    AfxMessageBox(_T("Could not hook up GPS sensor event sink"), MB_OK | MB_ICONSTOP);
    EndDialog(IDCANCEL);
    return FALSE;
  }

  return TRUE;
}

#pragma warning(suppress: 26434)
void CMFCSensorAppDlg::OnPaint()
{
  if (IsIconic())
  {
    CPaintDC dc{this};

#pragma warning(suppress: 26490)
    SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

    //Center icon in client rectangle
    const int cxIcon{GetSystemMetrics(SM_CXICON)};
    const int cyIcon{GetSystemMetrics(SM_CYICON)};
    CRect rect;
    GetClientRect(&rect);
    const int x{(rect.Width() - cxIcon + 1) / 2};
    const int y{(rect.Height() - cyIcon + 1) / 2};

    //Draw the icon
    dc.DrawIcon(x, y, m_hIcon);
  }
  else
    __super::OnPaint();
}

#pragma warning(suppress: 26434 26440)
HCURSOR CMFCSensorAppDlg::OnQueryDragIcon()
{
#pragma warning(suppress: 26473)
  return static_cast<HCURSOR>(m_hIcon);
}
